;;; init-use-package.el --- Get started with use-package in emacs
;; http://cachestocaches.com/2015/8/getting-started-use-package/
;http://github.com/CachesToCaches/getting_started_with_use_package/blob/master/init-use-package.el
;; Copyright (C) 2015 Gregory J Stein
;; Author: Gregory J Stein <gregory.j.stein@gmail.com>; Henrik Alfken <henke.uu.se@inbox.lv>
;; Code inspired by: http://stackoverflow.com/a/10093312/3672986
;;   https://swsnr.de/blog/2015/01/06/my-emacs-configuration-with-use-package/
;;   https://github.com/jwiegley/use-package
;;; Commentary:
;; As Sebastian Wiesner from http://www.lunaryorn.com/ points out, there is a "chicken and egg"
;; problem with use-package, which is capable of automatically downloading and installing
;; packages, but otherwise needs to be downloaded and installed manually. I include the
;; following code in my Emacs initialization file so that this process is automatic.
;;; Code:
;; Update package-archive lists:
(require 'package) (setq package-enable-at-startup nil)
(add-to-list 'package-archives '("gnu" . "http://elpa.gnu.org/packages/"))
(add-to-list 'package-archives '("melpa" . "http://melpa.org/packages/"))
(add-to-list 'package-archives '("marmalade" . "http://marmalade-repo.org/packages/"))
(package-initialize)
;(package-refresh-contents) ; !Uncomment this line if use-package gives an Error: "Not Found"
;; https://stackoverflow.com/questions/24833964/package-listed-in-melpa-but-not-found-in-package-install#29017473
(unless (package-installed-p 'use-package) (package-refresh-contents)
 (package-install 'use-package)) ; <<^^ Install 'use-package' if necessary.
(eval-when-compile (require 'use-package)) ;(require 'diminish) ; << Maybe uncomment.
(require 'bind-key) ; << If you use any :bind variant.
;;; init-use-package.el ends here
