;;; package --- Imperative and Object-Oriented Programming Methodology
;; Uppsala University - 2018 Autumn, Copyright © 2018 Henrik Alfken(Schulze)
;; This is NOT the official Emacs init file for the course IOOPM.
;;; Commentary:
;; ! Do you get "Error (use-package)"? Try <META-x> package-refresh-contents! Then restart eamcs.
;; Inspired by: http://jamiecollinson.com/blog/my-emacs-config/
;;http://homepages.inf.ed.ac.uk/s0243221/emacs/
;;https://github.com/anschwa/emacs.d
;; I like to set "(defcustom cua-prefix-override-inhibit-delay 0.04" which requires recompile:
; alt+x ; byte-compile-file RET ~/share/emacs/25.3/lisp/emulation/cua-base.el ; (after deletion)
; Also: (define-key isearch-mode-map "\C-v" 'isearch-yank-kill) ; BUG FIX! Line 1056.
; ^^ http://emacs.stackexchange.com/questions/14336/yanking-after-isearch-forward-backward#14390
;; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
;;; Code:
;http://emacs.stackexchange.com/questions/1/are-there-any-advantages-to-using-emacs-d-init-el-instead-of-emacs#6
; M-x eval-region ; <-- to evaluate selected code
; ^^ http://emacs.stackexchange.com/questions/21716/how-to-re-evaluate-and-apply-init-el#21718
; (byte-recompile-directory "~/share/emacs/" 0) ; Remove this line?
(byte-recompile-directory "~/.emacs.d/elpa/" 0)
;http://stackoverflow.com/questions/1217180/how-do-i-byte-compile-everything-in-my-emacs-d-directory#8902202
(setq package--init-file-ensured t)
;http://www.reddit.com/r/emacs/comments/56fvgd/is_there_a_way_to_stop_emacs_from_adding_the/
(defvar whitespace-line-column) (setq whitespace-line-column 116)
(global-linum-mode t) (setq column-number-mode t) ;(global-whitespace-mode 1)
;http://stackoverflow.com/questions/6378831/emacs-globally-enable-whitespace-mode#6379539
(setq mouse-wheel-scroll-amount '(1 ((shift) . 1))) (setq mouse-wheel-progressive-speed t)
(setq-default smooth-scroll-margin 0) (setq scroll-step 1 scroll-margin 1 scroll-conservatively 99)
;http://stackoverflow.com/questions/3631220/fix-to-get-smooth-scrolling-in-emacs#4160949
(setq inhibit-startup-message t) ;(setq initial-scratch-message "")
(fset 'yes-or-no-p 'y-or-n-p) (scroll-bar-mode -1) (tool-bar-mode -1) ;(menu-bar-mode -1)
;^^ http://emacs.stackexchange.com/questions/23773/disable-scrollbar-on-new-frame/23774#23774
(defvar display-time-24hr-format) (setq display-time-day-and-date t display-time-24hr-format t)
 (display-time) (show-paren-mode 1)
;^^ http://www.gnu.org/software/emacs/manual/html_node/efaq/Matching-parentheses.html
(setq-default major-mode 'text-mode) (setq sentence-end-double-space nil)
;http://stackoverflow.com/questions/18172728/the-difference-between-setq-and-setq-default-in-emacs-lisp#18173666
(prefer-coding-system 'utf-8) (set-language-environment "UTF-8") (setq delete-selection-mode 1)
;;http://ergoemacs.org/emacs_manual/emacs/Major-Modes.html#index-major_002dmode
;;http://emacs.stackexchange.com/questions/32205/delete-selection-mode-not-a-default#32208
(defvar imenu-auto-rescan) (setq imenu-auto-rescan t) (setq-default set-mark-command-repeat-pop t)
(global-auto-revert-mode 1) ; Automatically reload the file if it was modified by other program.
;^^ http://www.emacswiki.org/emacs/AutoRevertMode
(make-directory "~/.emacs.d/files-autosaved/" t)
;^ http://www.reddit.com/r/emacs/comments/8jrp5a/how_to_check_if_a_list_of_directories_exist_and
(setq auto-save-file-name-transforms '((".*" "~/.emacs.d/files-autosaved/" t))
 backup-by-copying t  backup-directory-alist '(("." . "~/.emacs.d/file-backups/"))
 delete-old-versions t  kept-new-versions 20  kept-old-versions 10  version-control t
 large-file-warning-threshold 1000000)
;http://emacs.stackexchange.com/questions/17210/how-to-place-all-auto-save-files-in-a-directory
;; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
;;; init-use-package.el --- Get started with use-package in emacs!
;;http://cachestocaches.com/2015/8/getting-started-use-package/
;http://github.com/CachesToCaches/getting_started_with_use_package/blob/master/init-use-package.el
;; Copyright (C) 2015 Gregory J Stein
;; Author: Gregory J Stein <gregory.j.stein@gmail.com>; Henrik Alfken <henke.uu.se@inbox.lv>
;; Code inspired by: http://stackoverflow.com/a/10093312/3672986
;;   http://swsnr.de/blog/2015/01/06/my-emacs-configuration-with-use-package/
;;   http://github.com/jwiegley/use-package
;;; Commentary:
;; As Sebastian Wiesner from http://www.lunaryorn.com/ points out, there is a "chicken and egg"
;; problem with use-package, which is capable of automatically downloading and installing
;; packages, but otherwise needs to be downloaded and installed manually. I include the
;; following code in my Emacs initialization file so that this process is automatic.
;;; Code:
;; Update package-archive lists:
(require 'package) (setq package-enable-at-startup nil)
(add-to-list 'package-archives '("melpa" . "http://melpa.org/packages/"))
; (add-to-list 'package-archives '("marmalade" . "https://marmalade-repo.org/packages/"))
(add-to-list 'package-archives '("gnu" . "http://elpa.gnu.org/packages/"))
(package-initialize)
;(package-refresh-contents) ; !Uncomment this line if use-package gives an Error: "Not Found"
;http://stackoverflow.com/questions/24833964/package-listed-in-melpa-but-not-found-in-package-install#29017473
(unless (package-installed-p 'use-package) (package-refresh-contents)
 (package-install 'use-package)) ; <<^^ Install 'use-package' if necessary.
(eval-when-compile (require 'use-package)) ;(require 'diminish) ; << Maybe uncomment.
; (require 'bind-key) ; << If you use any :bind variant.
;;; init-use-package.el ends here
;; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
(setq default-frame-alist '((left . -0) (width . 83) (fullscreen . fullheight)
 (font . "DejaVu Sans Mono-8") (vertical-scroll-bars . nil)))
(add-to-list 'default-frame-alist '(background-color . "#0f0f0f"))
;http://emacs.stackexchange.com/questions/31058/how-to-make-size-of-emacs-frame-persistent#31069
;;http://www.reddit.com/r/emacs/comments/9c0a4d/tip_setting_initial_frame_size_and_position/#t1_e573owh
; ^^ http://www.gnu.org/software/emacs/manual/html_node/emacs/Fonts.html#index-fontconfig-1667
;http://stackoverflow.com/questions/11418648/how-to-set-default-emacs-background-and-foreground-colors#11419691
;http://emacs.stackexchange.com/questions/7481/how-to-evaluate-the-variables-before-adding-them-to-a-list#7487
(use-package monokai-theme :ensure t :defer t :init (load-theme 'monokai t))
 (setq initial-frame-alist '((background-color . "#0f0f0f")))
; IF theme in "~/share/emacs/2x.x/etc/themes" - replace ":ensure t" by ":ensure nil"! /Alfken
;^^http://emacs.stackexchange.com/questions/10396/difference-between-init-and-config-in-use-package#10403
(use-package auto-complete :ensure t) (ac-config-default)
(require 'auto-complete) (require 'auto-complete-config);;
;http://emacs.stackexchange.com/questions/14505/how-to-make-cedet-auto-completion-more-detailed
;(use-package counsel :ensure t :bind ("M-x" . counsel-M-x))
;^^DON'T USE counsel! - "Wrong number of arguments: setq, 1"
(use-package crux :ensure t)
(use-package erlang :ensure t)
;^^ If "File error" or "Not found", do install package erlang manually. Link on next line! vv
;http://unix.stackexchange.com/questions/20447/how-to-enter-erlang-mode-in-emacs#79141
(use-package expand-region :ensure t)
(use-package ggtags :ensure t)
(add-hook 'c-mode-common-hook (lambda () (when (derived-mode-p 'c-mode 'c++-mode 'java-mode)
 (ggtags-mode 1))))
(use-package flycheck :ensure t :init (global-flycheck-mode))
;http://github.com/flycheck/flycheck/blob/master/doc/user/installation.rst#user-content-use-package
 (add-hook 'c-mode-hook 'flycheck-mode) (add-hook 'java-mode-hook 'flycheck-mode)
;http://github.com/flycheck/flycheck/blob/master/doc/user/installation.rst#user-content-use-package
(use-package htmlize :ensure t)
;http://github.com/jcf/emacs.d/blob/master/init-packages.org#user-content-htmlize
(use-package idomenu :ensure t)
(use-package iedit :ensure t) ;http://www.emacswiki.org/emacs/Iedit
(use-package ido :ensure t) (ido-mode) (setq ido-auto-merge-delay-time 9999) (require 'ido)
(use-package imenu-anywhere :ensure t)
(use-package saveplace :ensure t)
 (setq save-place-file "~/.emacs.d/saveplace") (setq-default save-place t)
(use-package smex :ensure t)
;http://github.com/jwiegley/use-package/issues/384
(use-package undo-tree :ensure t :init (global-undo-tree-mode))
;https://cestlaz.github.io/posts/using-emacs-16-undo-tree/
(use-package uniquify)
;;http://github.com/jwiegley/use-package/issues/320#issuecomment-186178559
(use-package use-package-chords :ensure t :config (key-chord-mode 1))
(use-package visual-regexp :ensure t)
(use-package which-key :ensure t :config (add-hook 'after-init-hook 'which-key-mode))
(use-package yasnippet :ensure t)
(use-package yasnippet-snippets :ensure t) (yas-global-mode 1)
;http://github.com/AndreaCrotti/yasnippet-snippets#user-content-from-melpa
;; - This is NOT the official Emacs init file for the course IOOPM. - ;;
;; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
;; vv More key-bindings starts here.   "(+shift)" refers to a swedish keyboard, not a US dito.
;; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
;http://stackoverflow.com/questions/23643272/modifying-emacs-isearch-key-bindings-to-yank#23647148
;http://github.com/anschwa/emacs.d#user-content-keybindings ; M-x describe-personal-keybindings
;; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
;; vv EMACS shortcut and command vv            vv Shortcut-key vv     In a normal editor* called
;; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
;http://ergoemacs.org/emacs/keystroke_rep.html   ; F1 k; (<--key) ;/; F1 w ; (<--command) ;
(windmove-default-keybindings 'meta)
; ^^ http://emacsblog.org/2008/05/01/quick-tip-easier-window-switching-in-emacs/
(bind-key* [home] 'crux-move-beginning-of-line)   ; [home] (the key [home] on your keyboard)
;http://fr.wikibooks.org/wiki/Emacs/bind-key ; bind-key*
(defvar sh-basic-offset) (setq sh-basic-offset 2)
(setq-default tab-width 2) (setq-default indent-tabs-mode nil) (setq tab-always-indent nil)
;^^ http://stackoverflow.com/questions/9741931/primitive-indentation-in-emacs#9743739
(add-hook 'prog-mode-hook (lambda () (local-set-key (kbd "TAB") #'tab-to-tab-stop)))
;http://www.reddit.com/r/emacs/comments/7nzltv/changing_tab_key_to_use_spaces/
(bind-key* (kbd "S-<tab>") 'shift-tab-spaces)
 (defun shift-tab-spaces () (interactive) (save-excursion (save-match-data
  (beginning-of-line) (when (looking-at "^\\s-+") (untabify (match-beginning 0)
   (match-end 0))) (when (looking-at "^  ") (replace-match "")))))
;http://stackoverflow.com/questions/2249955/emacs-shift-tab-to-left-shift-the-block#2250155
(bind-key* "\C-a" 'mark-whole-buffer)             ; ctrl+a           "Select all"
; ^^ http://ergoemacs.org/emacs/gnu_emacs_keybinding.txt (line 937)
(bind-key* "M-a" 'back-to-indentation)            ; alt+a            (Almost like [home])
(bind-key* "\C-b" 'describe-bindings)             ; ctrl+b           ( Settings | Shortcuts... )
(cua-mode t)      ;http://cua.dk/cua.html        ; ctrl+c           "Copy"
(bind-key* "\C-d" 'duplicate-line)                ; ctrl+d           "Duplicate Line"
 (defun duplicate-line() (interactive) (kill-whole-line) (yank) (yank))
(bind-key* "\C-f" 'find-forward)                  ; ctrl+f           "Find..."
 (defun find-forward() (interactive) (call-interactively 'isearch-forward-regexp))
 ;http://stackoverflow.com/questions/13095405/isearch-region-if-active-emacs
 (define-key isearch-mode-map "\C-f" 'isearch-repeat-forward)
;http://stackoverflow.com/questions/7411920/how-to-bind-search-and-search-repeat-to-c-f-in-emacs#7412099
 (add-hook 'isearch-mode-hook #'find-selected)
  (defun find-selected() (when mark-active (let ((region (funcall region-extract-function nil)))
   (deactivate-mark) (isearch-update) (isearch-yank-string region))))
;http://stackoverflow.com/questions/202803/searching-for-marked-selected-text-in-emacs#32002122
  (bind-key* "<f3>" 'isearch-repeat-forward)
  (bind-key* (kbd "S-<f3>") 'isearch-repeat-backward)
(bind-key* "M-S-\C-f" 'indent-for-tab-command)    ; alt+ctrl+shift+f "Format Selected Code"
(bind-key* "M-\C-f" 'format-file)                 ; alt+ctrl+f       "Format Whole File"
 (defun format-file() (interactive) (call-interactively 'mark-whole-buffer)
  (call-interactively 'indent-for-tab-command))
;http://emacs.stackexchange.com/questions/36944/key-binding-to-invoke-more-than-two-commands#36946
(bind-key* "M-h" 'er/expand-region)               ; alt+h            (expand region)
(bind-key* "\C-h" 'vr/query-replace)              ; ctrl+h           "Replace..."
(bind-key* "M-i" 'idomenu)                        ; alt+i            (imenu)
(bind-key* "\C-k" 'comment)                       ; ctrl+k           "Comment"
 (defun comment() (interactive) (let (beg end) (if (region-active-p) (setq beg
  (region-beginning) end (region-end)) (setq beg (line-beginning-position) end
  (line-end-position))) (comment-region beg end)))
(bind-key* "S-\C-k" 'uncomment)                   ; ctrl+alt+shift+d "Uncomment"
 (defun uncomment() (interactive)  (let (beg end) (if (region-active-p) (setq beg
  (region-beginning) end (region-end)) (setq beg (line-beginning-position) end
  (line-end-position))) (uncomment-region beg end)))
(bind-key* "M-m" 'switch-to-minibuffer)           ; alt+m            (switch to minibuffer)
 (defun switch-to-minibuffer() (interactive) (when (active-minibuffer-window)
  (select-frame-set-input-focus (window-frame (active-minibuffer-window)))
  (select-window (active-minibuffer-window))))
;http://superuser.com/questions/132225/how-to-get-back-to-an-active-minibuffer-prompt-in-emacs-without-the-mouse
(bind-key* "\C-n" 'make-frame)                    ; ctrl+n           "New Window"
(bind-key* "M-n" 'flycheck-next-error)            ; alt+n            - Next Flycheck error
(bind-key* "\C-o" 'find-file)                     ; ctrl+o           "Open..."
(bind-key* "S-\C-o" 'find-file-other-frame)       ; ctrl+shift+o  "Open in New Window"
 ; (defun new-window-open () (interactive) (make-frame) (find-file))
(bind-key* "M-o" 'other-window)                   ; alt+o            (switch window)
;;http://www.reddit.com/r/emacs/comments/6dya5h/are_there_any_default_bindings_worth_rebinding/
(bind-key* "M-p" 'flycheck-previous-error)        ; alt+p            - Previous Flycheck error
(bind-key* "\C-s" 'save-buffer)                   ; ctrl+s           "Save"
(bind-key* "S-\C-s" 'save-some-buffers)           ; ctrl+shift+s!    "Save All"
(bind-key* "M-\C-s" 'write-file)                  ; alt+ctrl+s       "Save As..."
(bind-key* "\C-t" 'toggle-comment)                ; ctrl+t           "Toggle comment"
 (defun toggle-comment() (interactive) (let (beg end) (if (region-active-p) (setq beg
  (region-beginning) end (region-end)) (setq beg (line-beginning-position) end
  (line-end-position))) (comment-or-uncomment-region beg end)))
;http://stackoverflow.com/questions/9688748/emacs-comment-uncomment-current-line#9697222
(cua-mode t)      ;http://cua.dk/cua.html        ; ctrl+v           "Paste"
(bind-key* "\C-w" 'kill-this-buffer)              ; ctrl+w           "Close"
;;http://superuser.com/questions/354849/emacs-kill-buffer-without-prompt#354878
(bind-key* "M-x" 'smex)
(cua-mode t)      ;http://cua.dk/cua.html        ; ctrl+x           "Cut"
(bind-key* "\C-x0" 'close-window)                 ; ctrl+x, 0        (Emacs: close window)
 (defun close-window () (interactive) (condition-case nil (delete-window)
  (error (save-buffers-kill-terminal))))
(bind-key* "\C-z" 'undo-tree-undo)                ; ctrl+z           "Undo"
(bind-key* "S-\C-z" 'undo-tree-redo)              ; ctrl+shift+z     "Redo"
(bind-key* "M-S-<backspace>" 'kill-whole-line)    ; alt+shift+BckSpc "Cut current line"
(bind-key* (kbd "C-.") 'imenu-anywhere)           ; ctrl+.           (Language-aware navigation)
(bind-key* (kbd "C-;") 'iedit-mode)               ; ctrl(+shift)+;   "Multiple Cursors" (simple)
; help-command ; (Enabled by default)            ; f1               "Help"
(bind-key* "M-/" 'hippie-expand)                  ; alt(+shift)+/    (expand if possible)
 (setq hippie-expand-try-functions-list '(try-expand-dabbrev try-expand-dabbrev-all-buffers
 try-complete-file-name-partially try-complete-file-name try-expand-all-abbrevs try-expand-list
 try-expand-dabbrev-from-kill try-expand-line try-complete-lisp-symbol-partially
 try-complete-lisp-symbol))
(bind-key* "M-<f4>" 'close-frame)                 ; alt+f4           "Exit/Close"
 (defun close-frame () (interactive)
  (condition-case nil (delete-frame) (error (save-buffers-kill-terminal))))
;http://emacs.stackexchange.com/questions/13485/something-like-delete-frame-but-that-would-also-delete-the-last-frame-like-al
(bind-key* [escape] 'keyboard-clean)
 (defun keyboard-clean () (interactive) (iedit-quit) (isearch-done) (keyboard-escape-quit))
 ;http://stackoverflow.com/questions/12585670/relationship-between-emacs-functions-and-commands#12586078
(define-key minibuffer-local-map [escape] 'abort-recursive-edit)
;http://superuser.com/questions/795763/how-to-make-emacs-quit-the-minibuffer-with-one-press-of-esc#796225
;; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
;; * Examples of what I mean by "normal editor" include:
;;   Kate, Noteapad++, GEdit, Notepad2, Crimson Editor, VS Code
;; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
;; ^^ More key-bindings ends here. ^^
;; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
;; - This is NOT the official Emacs init file for the course IOOPM. - ;;

; Settings for the C programming language:
(add-hook 'c-mode-common-hook (lambda() (progn (setq comment-start "//") (setq comment-end ""))))

(setq kill-buffer-query-functions (delq 'process-kill-buffer-query-function kill-buffer-query-functions))
;http://emacs.stackexchange.com/questions/14509/kill-process-buffer-without-confirmation#14511
(setq custom-file "~/.emacs.d/custom.el") (load custom-file 'noerror)
; ^^ http://emacsblog.org/2008/12/06/quick-tip-detaching-the-custom-file/
(add-hook 'before-save-hook 'do-before-save)
 (defun do-before-save()(delete-trailing-whitespace))
;http://stackoverflow.com/questions/40179200/how-to-use-function-write-region#40179637
(add-hook 'focus-out-hook 'save-buffer)
; ^^ http://emacs.stackexchange.com/questions/265/how-to-auto-save-buffers-when-emacs-loses-focus#266
;; - This is NOT the official Emacs init file for the course IOOPM. - ;;
;TODO? Add use-pakage for haskell-mode and js2-mode ??
;;; init.el ends here
